
/*=============================================================================
   rtf2htm, a command-line program to convert RTF-formatted documents to HTML.
   Copyright (C) 2000,2001 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at tuorfa@yahoo.com.
=============================================================================*/


/*----------------------------------------------------------------------
 * Module name:    attr
 * Author name:    Zach Smith
 * Create date:    1 Aug 2001
 * Last change:    5 Aug 2001
 * Purpose:        Character attribute stack.
 *----------------------------------------------------------------------
 * Changes:
 * 01 Aug 01, tuorfa@yahoo.com: moved code over from convert.c
 * 06 Aug 01, tuorfa@yahoo.com: added several font attributes.
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "malloc.h"
#include "defs.h"
#include "error.h"
#include "attr.h"
#include "main.h"


extern void starting_body();
extern void starting_text();

extern int simulate_allcaps;
extern int simulate_smallcaps;


/* XX convert to linked list in next version */
#define MAX_ATTRS (1000)
static unsigned char attr_stack [MAX_ATTRS];
static char *attr_stack_params [MAX_ATTRS];
static int tos=-1;



void attr_push_core (int attr, char* param) {
	switch(attr) 
	{
	case ATTR_BOLD: printf ("<b>"); break;
	case ATTR_ITALIC: printf ("<i>"); break;

	/* Various underlines, they all resolve to HTML's <u> */
	case ATTR_THICK_UL:
	case ATTR_WAVE_UL:
	case ATTR_DASH_UL:
	case ATTR_DOT_UL: 
	case ATTR_DOT_DASH_UL:
	case ATTR_2DOT_DASH_UL:
	case ATTR_DOUBLE_UL: 
	case ATTR_WORD_UL: 
	case ATTR_UNDERLINE: printf ("<u>"); break;

	case ATTR_FONTSIZE: 
		printf ("<span style=\"font-size:%s\">\n",param); 
		break;
	case ATTR_STD_FONTSIZE: 
		printf ("<font size=%s>\n",param); 
		break;

	case ATTR_FONTFACE: printf ("<font face=\"%s\">\n",param); break;

	case ATTR_FOREGROUND: printf ("<font color=%s>\n", param); break;
	case ATTR_BACKGROUND: 
		if (!simple_mode)
		  printf ("<span style=\"background:%s\">\n",param); 
		break;

	case ATTR_SUPER: printf ("<sup>"); break;
	case ATTR_SUB: printf ("<sub>"); break;

	case ATTR_STRIKE: 
	case ATTR_DBL_STRIKE: 
		printf ("<s>"); break;

	case ATTR_EXPAND: 
		if (!simple_mode) {
			printf ("<span style=\"letter-spacing: %s", param);
			printf ("pt\">\n");
		} else {
			printf ("<!--begin expand/condense-->"); 
		}
		break;

	case ATTR_OUTLINE: printf ("<!--begin outline-->"); 
		printf ("<b>");
		break;
	case ATTR_SHADOW: printf ("<!--begin shadow-->"); 
		printf ("<b>");
		break;
	case ATTR_EMBOSS: printf ("<!--begin emboss-->"); 
		if (!simple_mode)
			printf ("<span style=\"background:#d0d0d0\">\n"); 
		else
			printf ("<i>");
		break;
	case ATTR_ENGRAVE: printf ("<!--begin engrave-->"); 
		if (!simple_mode)
			printf ("<span style=\"background:#d0d0d0\">\n"); 
		else
			printf ("<i>");
		break;

	case ATTR_CAPS:
		simulate_allcaps = TRUE;
		break;

	case ATTR_SMALLCAPS: 
		simulate_smallcaps = TRUE;
		break;

	case ATTR_MARKER: break;
	}
}

void attr_pop_core (int attr, char* param) {
	switch(attr) 
	{
	case ATTR_BOLD: printf ("</b>"); break;
	case ATTR_ITALIC: printf ("</i>"); break;

	/* Various underlines, they all resolve to HTML's </u> */
	case ATTR_THICK_UL:
	case ATTR_WAVE_UL:
	case ATTR_DASH_UL:
	case ATTR_DOT_UL: 
	case ATTR_DOT_DASH_UL:
	case ATTR_2DOT_DASH_UL: 
	case ATTR_DOUBLE_UL: 
	case ATTR_WORD_UL: 
	case ATTR_UNDERLINE: printf ("</u>"); break;

	case ATTR_FONTSIZE: 
		if (simple_mode)
			printf ("</font>"); 
		else
			printf ("</span>"); 
		break;
	case ATTR_STD_FONTSIZE: 
		printf ("</font>"); 
		break;

	case ATTR_FONTFACE: printf ("</font>"); break;

	case ATTR_FOREGROUND: printf ("</font>"); break;
	case ATTR_BACKGROUND: 
		if (!simple_mode)
		  printf ("</span>"); 
		break;

	case ATTR_SUPER: printf ("</sup>"); break;
	case ATTR_SUB: printf ("</sub>"); break;

	case ATTR_STRIKE: 
	case ATTR_DBL_STRIKE: 
		printf ("</s>"); break;

	case ATTR_OUTLINE: printf ("<!--end outline-->"); 
		printf ("</b>");
		break;
	case ATTR_SHADOW: printf ("<!--end shadow-->"); 
		printf ("</b>");
		break;
	case ATTR_EMBOSS: printf ("<!--end emboss-->"); 
		if (!simple_mode)
			printf ("</span>"); 
		else
			printf ("</i>");
		break;
	case ATTR_ENGRAVE: printf ("<!--end engrave-->"); 
		if (!simple_mode)
			printf ("</span>"); 
		else
			printf ("</i>");
		break;

	case ATTR_EXPAND: 
		if (!simple_mode) {
			printf ("</span>\n");
		} else {
			printf ("<!--end expand/condense-->");
		}
		break;

	case ATTR_CAPS:
		simulate_allcaps = FALSE;
		break;

	case ATTR_SMALLCAPS:
		simulate_smallcaps = FALSE;
		break;

	case ATTR_MARKER: break;
	}
	printf ("\n");
}

void attr_push(int attr, char* param) 
{
	if (tos>=MAX_ATTRS) { fprintf (stderr,"Too many attributes!\n"); return; }

	/* If the attribute refers to text, we're in the <body> section. */
	if (attr != ATTR_MARKER) {
		starting_body();
		starting_text();
	}

	++tos;
	attr_stack [tos]=attr;
	if (param) 
		attr_stack_params [tos]=my_strdup(param);
	else
		attr_stack_params [tos]=NULL;

	attr_push_core (attr, param);
}

void attr_push_mark() {
	attr_push(ATTR_MARKER,NULL);
}

int attr_pop(int attr) {
	if(tos>=0 && attr_stack[tos]==attr)
	{
		int attr = attr_stack [tos];
		char *param = attr_stack_params [tos];
		attr_pop_core (attr, param);
		if (param) my_free(param);
		tos--;
		return TRUE;
	}
	else
		return FALSE;
}

int attr_read() {
	if(tos>=0)
	{
		int attr = attr_stack [tos];
		return attr;
	}
	else
		return ATTR_NONE;
}

void attr_drop_all () {
	while (tos>=0 && attr_stack[tos]!=ATTR_MARKER) {
		char *param=attr_stack_params [tos];
		if (param) my_free(param);
		tos--;
	}
}

void attr_pop_mark() {
	if (!attr_pop(ATTR_MARKER))
		warning_handler ("cannot find ATTR_MARKER on attribute stack");
}

/* Routine to undo all attributes heretofore applied, reversing
 * the order in which they were applied.
 */
void attr_pop_all() {
	while (tos>=0 && attr_stack[tos]!=ATTR_MARKER) {
		int attr=attr_stack [tos];
		char *param=attr_stack_params [tos];
		if (param) my_free(param);
		attr_pop_core (attr, param);
		tos--;
	}
}

/* Routine to print out HTML to undo all attributes heretofore applied, 
 * but which actually leves it all on the stack. This is needed for \cell,
 * often occurs within a group, yet HTML uses <td></td> which in theory
 * will remove attributes at the end of that block.
 */
void attr_pop_dump() {
	int i=tos;
	while (i>=0 && attr_stack[i]!=ATTR_MARKER) {
		int attr=attr_stack [i];
		char *param=attr_stack_params [i];
		attr_pop_core (attr, param);
		i--;
	}
}

/* Routine to print out HTML to *redo* all attributes heretofore applied, 
 * but which actually leves it all on the stack. This is needed for \cell,
 * often occurs within a group, yet HTML uses <td></td> which in theory
 * will remove attributes at the end of that block.
 */
void attr_push_dump() {
	int i=tos;

	/* First find the last marker */
	while (i>=0 && attr_stack[i]!=ATTR_MARKER)
		i--;

	/* Now print everything above the marker */
	i++;
	while (i<=tos && attr_stack[i]!=ATTR_MARKER) {
		char *param = attr_stack_params [i];
		int attr = attr_stack[i++];
		attr_push_core (attr, param);
		i++;
	}
}

