
/*=============================================================================
   rtf2htm, a command-line program to convert RTF-formatted documents to HTML.
   Copyright (C) 2000,2001 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at tuorfa@yahoo.com.
=============================================================================*/


/*----------------------------------------------------------------------
 * Module name:    special
 * Author name:    Zach Smith
 * Create date:    1 Aug 2001
 * Last change:    3 Aug 2001
 * Purpose:        Special characters conversion.
 *----------------------------------------------------------------------
 * Changes:
 * tuorfa@yahoo.com, 1 Aug 01, code moved over from convert.c
 * tuorfa@yahoo.com, 3 Aug 01, removed null entries to save space
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include "special.h"


static char* special_exprs_ansi [] = {
/* 0x90 */ NULL,"lsquo","rsquo","ldqyo","rdquo","bull","ndash","mdash",
/* 0x98 */ NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
/* 0xa0 */ "nbsp","iexcl","cent","pound","curren","yen","brvbar","sect",
/* 0xa8 */ "uml","copy","ordf","laquo","not","shy","reg","macr",
/* 0xb0 */ "plus","plusmn","sup2","sup3","acute","micro","para","middot",
/* 0xb8 */ "cedil","sup1","frac14","ordm","raquo","frac12","frac34","iquest",
/* 0xc0 */ "Agrave","Aacute","Acirc","Atilde","Auml","Aring","AElig","Ccedil",
/* 0xc8 */ "Egrave","Eacute","Ecirc","Euml","Igrave","Iacute","Icirc","Iuml",
/* 0xd0 */ "ETH","Ntilde","Ograve","Oacute","Ocirc","Otilde","Ouml","times",
/* 0xd8 */ "Oslash","Ugrave","Uacute","Ucirc","Uuml","Yacute","THORN","szlig",
/* 0xe0 */ "agrave","aacute","acirc","atilde","auml","aring","aelig","ccedil",
/* 0xe8 */ "egrave","eacute","ecirc","euml","igrave","iacute","icirc","iuml",
/* 0xf0 */ "eth","ntilde","ograve","oacute","ocirc","otilde","ouml","divide",
/* 0xf8 */ "oslash","ugrave","uacute","ucirc","uuml","yacute","thorn","yuml",
};

static char* special_exprs_mac [] = {
#if 0
/* 0xa4 */ "bull"
#endif
/* 0xd0 */ "mdash","ndash","ldquo","rdquo","lquo","rquo",
};

static char* special_exprs_cp437 [] = {
/* 0x80 */ "ccedil",
/* 0x81 */ "uuml",
/* 0x82 */ "eacute",
/* 0x83 */ "acirc",
/* 0x84 */ "auml",
/* 0x85 */ "agrave",
/* 0x86 */ "aring",
/* 0x87 */ "ccedil",
/* 0x88 */ "ecirc",
/* 0x89 */ "euml",
/* 0x8a */ "egrave",
/* 0x8b */ "iuml",
/* 0x8c */ "icirc",
/* 0x8d */ "igrave",
/* 0x8e */ "auml",
/* 0x8f */ "aring",
/* 0x90 */ "eacute",
/* 0x91 */ "aelig",
/* 0x92 */ "aelig",
/* 0x93 */ "ocirc",
/* 0x94 */ "ouml",
/* 0x95 */ "ograve",
/* 0x96 */ "ucirc",
/* 0x97 */ "ugrave",
/* 0x98 */ "yuml",
/* 0x99 */ "ouml",
/* 0x9a */ "uuml",
/* 0x9b */ "cent",
/* 0x9c */ "pound",
/* 0x9d */ "yen",
/* 0x9e */ "#8359", /* peseta */
/* 0x9f */ "#402", /* small f with hook */
/* 0xa0 */ "aacute",
/* 0xa1 */ "iacute",
/* 0xa2 */ "oacute",
/* 0xa3 */ "uacute",
/* 0xa4 */ "ntilde",
/* 0xa5 */ "ntilde",
/* 0xa6 */ "ordf",
/* 0xa7 */ "frac14",
/* 0xa8 */ "iquest",
/* 0xa9 */ "#8976", /* reversed not */
/* 0xaa */ "not",
/* 0xab */ "frac12",
/* 0xac */ "raquo",
/* 0xad */ "iexcl",
/* 0xae */ "laquo",
/* 0xaf */ "ordm",
/* 0xb0 */ "#9617", /* light shade */
/* 0xb1 */ "#9618", /* med. shade */
/* 0xb2 */ "#9619", /* dark shade */
/* 0xb3 */ "#9474", /* box-draw light vert. */
/* 0xb4 */ "#9508", /* box-draw light vert. + lt. */
/* 0xb5 */ "#9569", /* box-draw vert. sgl. + lt. dbl. */
/* 0xb6 */ "#9570", /* box-draw vert. dbl. + lt. sgl. */
/* 0xb7 */ "#9558", /* box-draw dn. dbl. + lt. sgl. */
/* 0xb8 */ "#9557", /* box-draw dn. sgl. + lt. dbl. */
/* 0xb9 */ "#9571", /* box-draw dbl. vert. + lt. */
/* 0xba */ "#9553", /* box-draw dbl. vert. */
/* 0xbb */ "#9559", /* box-draw dbl. dn. + lt. */
/* 0xbc */ "#9565", /* box-draw dbl. up + lt. */
/* 0xbd */ "#9564", /* box-draw up dbl. + lt. sgl. */
/* 0xbe */ "#9563", /* box-draw up sgl. + lt. dbl. */
/* 0xbf */ "#9488", /* box-draw light dn. + lt. */
/* 0xc0 */ "#9492", /* box-draw light up + rt. */
/* 0xc1 */ "#9524", /* box-draw light up + horiz. */
/* 0xc2 */ "#9516", /* box-draw light dn. + horiz. */
/* 0xc3 */ "#9500", /* box-draw light vert. + rt. */
/* 0xc4 */ "#9472", /* box-draw light horiz. */
/* 0xc5 */ "#9532", /* box-draw light vert. + horiz. */
/* 0xc6 */ "#9566", /* box-draw vert. sgl. + rt. dbl. */
/* 0xc7 */ "#9567", /* box-draw vert. dbl. + rt. sgl. */
/* 0xc8 */ "#9562", /* box-draw dbl. up + rt. */
/* 0xc9 */ "#9556", /* box-draw dbl. dn. + rt. */
/* 0xca */ "#9577", /* box-draw dbl. up + horiz. */
/* 0xcb */ "#9574", /* box-draw dbl. dn. + horiz. */
/* 0xcc */ "#9568", /* box-draw dbl. vert. + rt. */
/* 0xcd */ "#9552", /* box-draw dbl. horiz. */
/* 0xce */ "#9580", /* box-draw dbl. vert. + horiz. */
/* 0xcf */ "#9575", /* box-draw up sgl. + horiz. dbl. */
/* 0xd0 */ "#9576", /* box-draw up dbl. + horiz. sgl. */
/* 0xd1 */ "#9572", /* box-draw dn. sgl. + horiz. dbl. */
/* 0xd2 */ "#9573", /* box-draw dn. dbl. + horiz. sgl. */
/* 0xd3 */ "#9561", /* box-draw up dbl. + rt. sgl. */
/* 0xd4 */ "#9560", /* box-draw up sgl. + rt. dbl. */
/* 0xd5 */ "#9554", /* box-draw dn. sgl. + rt. dbl. */
/* 0xd6 */ "#9555", /* box-draw dn. dbl. + rt. sgl. */
/* 0xd7 */ "#9579", /* box-draw vert. dbl. + horiz. sgl. */
/* 0xd8 */ "#9578", /* box-draw vert. sgl. + horiz. dbl. */
/* 0xd9 */ "#9496", /* box-draw light up + lt. */
/* 0xda */ "#9484", /* box-draw light dn. + rt. */
/* 0xdb */ "#9608", /* full block */
/* 0xdc */ "#9604", /* lower 1/2 block */
/* 0xdd */ "#9612", /* lt. 1/2 block */
/* 0xde */ "#9616", /* rt. 1/2 block */
/* 0xdf */ "#9600", /* upper 1/2 block */
/* 0xe0 */ "#945", /* greek small alpha */
/* 0xe1 */ "szlig",
/* 0xe2 */ "#915", /* greek cap gamma */
/* 0xe3 */ "#960", /* greek small pi */
/* 0xe4 */ "#931", /* greek cap sigma */
/* 0xe5 */ "#963", /* greek small sigma */
/* 0xe6 */ "micro",
/* 0xe7 */ "#964", /* greek small tau */
/* 0xe8 */ "#934", /* greek cap phi */
/* 0xe9 */ "#920", /* greek cap theta */
/* 0xea */ "#937", /* greek cap omega */
/* 0xeb */ "#948", /* greek small delta */
/* 0xec */ "#8734", /* inf. */
/* 0xed */ "#966", /* greek small phi */
/* 0xee */ "#949", /* greek small epsilon */
/* 0xef */ "#8745", /* intersect */
/* 0xf0 */ "#8801", /* identical */
/* 0xf1 */ "plusmn",
/* 0xf2 */ "#8805", /* greater-than or equal to */
/* 0xf3 */ "#8804", /* less-than or equal to */
/* 0xf4 */ "#8992", /* top 1/2 integral */
/* 0xf5 */ "#8993", /* bottom 1/2 integral */
/* 0xf6 */ "divide",
/* 0xf7 */ "#8776", /* almost = */
/* 0xf8 */ "plus",
/* 0xf9 */ "#8729", /* bullet op */
/* 0xfa */ "middot",
/* 0xfb */ "#8730", /* sqrt */
/* 0xfc */ "#8319", /* super-script small n */
/* 0xfd */ "sup2",
/* 0xfe */ "#9632", /* black square */
/* 0xff */ "nbsp",
};

static char* special_exprs_cp850 [] = {
/* 0x80 */  "ccedil",
/* 0x81 */  "uuml",
/* 0x82 */  "eacute",
/* 0x83 */  "acirc",
/* 0x84 */  "auml",
/* 0x85 */  "agrave",
/* 0x86 */  "aring",
/* 0x87 */  "ccedil",
/* 0x88 */  "ecirc",
/* 0x89 */  "euml",
/* 0x8a */  "egrave",
/* 0x8b */  "iuml",
/* 0x8c */  "icirc",
/* 0x8d */  "igrave",
/* 0x8e */  "auml",
/* 0x8f */  "aring",
/* 0x90 */  "eacute",
/* 0x91 */  "aelig",
/* 0x92 */  "aelig",
/* 0x93 */  "ocirc",
/* 0x94 */  "ouml",
/* 0x95 */  "ograve",
/* 0x96 */  "ucirc",
/* 0x97 */  "ugrave",
/* 0x98 */  "yuml",
/* 0x99 */  "ouml",
/* 0x9a */  "uuml",
/* 0x9b */  "oslash",
/* 0x9c */  "pound",
/* 0x9d */  "oslash",
/* 0x9e */  "times",
/* 0x9f */  "#402", /* small f with hook */
/* 0xa0 */  "aacute",
/* 0xa1 */  "iacute",
/* 0xa2 */  "oacute",
/* 0xa3 */  "uacute",
/* 0xa4 */  "ntilde",
/* 0xa5 */  "ntilde",
/* 0xa6 */  "ordf",
/* 0xa7 */  "frac14",
/* 0xa8 */  "iquest",
/* 0xa9 */  "reg",
/* 0xaa */  "not",
/* 0xab */  "frac12",
/* 0xac */  "raquo",
/* 0xad */  "iexcl",
/* 0xae */  "laquo",
/* 0xaf */  "ordm",
/* 0xb0 */  "#9617", /* light shade */
/* 0xb1 */  "#9618", /* med. shade */
/* 0xb2 */  "#9619", /* dark shade */
/* 0xb3 */  "#9474", /* box-draw light vert. */
/* 0xb4 */  "#9508", /* box-draw light vert. + lt. */
/* 0xb5 */  "aacute",
/* 0xb6 */  "acirc",
/* 0xb7 */  "agrave",
/* 0xb8 */  "copy",
/* 0xb9 */  "#9571", /* box-draw dbl. vert. + lt. */
/* 0xba */  "#9553", /* box-draw dbl. vert. */
/* 0xbb */  "#9559", /* box-draw dbl. dn. + lt. */
/* 0xbc */  "#9565", /* box-draw dbl. up + lt. */
/* 0xbd */  "cent",
/* 0xbe */  "yen",
/* 0xbf */  "#9488", /* box-draw light dn. + lt. */
/* 0xc0 */  "#9492", /* box-draw light up + rt. */
/* 0xc1 */  "#9524", /* box-draw light up + horiz. */
/* 0xc2 */  "#9516", /* box-draw light dn. + horiz. */
/* 0xc3 */  "#9500", /* box-draw light vert. + rt. */
/* 0xc4 */  "#9472", /* box-draw light horiz. */
/* 0xc5 */  "#9532", /* box-draw light vert. + horiz. */
/* 0xc6 */  "atilde",
/* 0xc7 */  "atilde",
/* 0xc8 */  "#9562", /* box-draw dbl. up + rt. */
/* 0xc9 */  "#9556", /* box-draw dbl. dn. + rt. */
/* 0xca */  "#9577", /* box-draw dbl. up + horiz. */
/* 0xcb */  "#9574", /* box-draw dbl. dn. + horiz. */
/* 0xcc */  "#9568", /* box-draw dbl. vert. + rt. */
/* 0xcd */  "#9552", /* box-draw dbl. horiz. */
/* 0xce */  "#9580", /* box-draw dbl. vert. + horiz. */
/* 0xcf */  "curren",
/* 0xd0 */  "eth",
/* 0xd1 */  "eth",
/* 0xd2 */  "ecirc",
/* 0xd3 */  "euml",
/* 0xd4 */  "egrave",
/* 0xd5 */  "#305", /* small dotless i */
/* 0xd6 */  "iacute",
/* 0xd7 */  "icirc",
/* 0xd8 */  "iuml",
/* 0xd9 */  "#9496", /* box-draw light up + lt. */
/* 0xda */  "#9484", /* box-draw light dn. + rt. */
/* 0xdb */  "#9608", /* full-block */
/* 0xdc */  "#9604", /* lower 1/2 block */
/* 0xdd */  "brvbar",
/* 0xde */  "igrave",
/* 0xdf */  "#9600", /* upper 1/2 block */
/* 0xe0 */  "oacute",
/* 0xe1 */  "szlig",
/* 0xe2 */  "ocirc",
/* 0xe3 */  "ograve",
/* 0xe4 */  "otilde",
/* 0xe5 */  "otilde",
/* 0xe6 */  "micro",
/* 0xe7 */  "thorn",
/* 0xe8 */  "thorn",
/* 0xe9 */  "uacute",
/* 0xea */  "ucirc",
/* 0xeb */  "ugrave",
/* 0xec */  "yacute",
/* 0xed */  "yacute",
/* 0xee */  "macr",
/* 0xef */  "acute",
/* 0xf0 */  "shy",
/* 0xf1 */  "plusmn",
/* 0xf2 */  "#8215", /* dbl. lowline */
/* 0xf3 */  "frac34",
/* 0xf4 */  "para",
/* 0xf5 */  "sect",
/* 0xf6 */  "divide",
/* 0xf7 */  "cedil",
/* 0xf8 */  "plus",
/* 0xf9 */  "uml",
/* 0xfa */  "middot",
/* 0xfb */  "sup1",
/* 0xfc */  "sup3",
/* 0xfd */  "sup2",
/* 0xfe */  "#9632", /* black square */
/* 0xff */  "nbsp",
};



char *
special_exprs_lookup (int which, int ch)
{
	char **ptr=NULL;

	switch (which) {
	case CHARSET_ANSI:
		if (ch < 0x90) return NULL;
		ptr = special_exprs_ansi;
		if (ptr) return ptr[ch-0x90];
		break;
	case CHARSET_MAC:
		if (ch == 0xa4) return "bull";
		if (ch < 0xd0 || ch >= 0xd6) return NULL;
		ptr = special_exprs_mac;
		if (ptr) return ptr[ch-0xa0];
		break;
	case CHARSET_CP437:
		ptr = special_exprs_cp437;
		if (ptr) return ptr[ch-0x80];
		break;
	case CHARSET_CP850:
		ptr = special_exprs_cp850;
		if (ptr) return ptr[ch-0x80];
		break;
	}
	return NULL;
}

